from package.data.init import ensure_db_ready, get_conn
from package.entity.project import Project


@ensure_db_ready
def add_project(project: Project):
    sql = "insert into projects(project_name,project_status,fatigue_rate,fatigue_total) values(?,?,?,?)"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (
                project.project_name,
                project.project_status,
                project.fatigue_rate,
                project.fatigue_total,
            ),
        )

    return cursor.lastrowid


@ensure_db_ready
def update_project_status(project: Project):
    sql = "update projects set project_status = ? where id = ?"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (
                project.project_status,
                project.id,
            ),
        )
        conn.commit()


@ensure_db_ready
def get_project_id(id: int):
    sql = "select id,project_name,fatigue_rate,fatigue_total,project_status from projects where id = ?"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (id,),
        )

        return Project(**dict(cursor.fetchone()))


@ensure_db_ready
def get_project_name(project_name):
    sql = "select id,project_name,fatigue_total,fatigue_rate,project_status from projects where project_name = ?"
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
            (project_name,),
        )
        row = cursor.fetchone()
        if not row:
            return None

        return Project(**dict(row))


@ensure_db_ready
def get_all_projects():
    sql = (
        "select id,project_name,fatigue_total,fatigue_rate,project_status from projects"
    )
    with get_conn() as conn:
        cursor = conn.cursor()
        cursor.execute(
            sql,
        )
        rows = cursor.fetchall()
        if not rows:
            return None

        results = []
        for r in rows:
            results.append(Project(**dict(r)))
        return results


# @ensure_db_ready
# def get_like_projects(keyname):
#     sql = "select id,project_name,fatigue_total,fatigue_rate,project_status from projects like ?%"
#     with get_conn() as conn:
#         cursor = conn.cursor()
#         cursor.execute(sql, (keyname,))
#         rows = cursor.fetchall()
#         if not rows:
#             return None

#         results = []
#         for r in rows:
#             results.append(Project(**dict(r)))
#         return results



from package.data.project import get_project_name
from package.data.record import (
    get_today_accumulated_fatigue,
)
from rich.console import Console
from rich.table import Table
from rich.progress_bar import ProgressBar
from rich.text import Text
import typer


def show(project_name):
    console = Console()
    project = get_project_name(project_name=project_name)

    if not project:
        typer.secho("找不到此项目", fg=typer.colors.RED, err=True)
        return

    cost = int(get_today_accumulated_fatigue(project.id))

    if cost > project.fatigue_total:
        cost = project.fatigue_total

    rest = project.fatigue_total - cost

    if rest < 0:
        rest = 0

    bar = ProgressBar(
        total=project.fatigue_total,
        completed=rest,
        width=40,
        complete_style="yellow",
        finished_style="green",
    )

    status = (
        Text(f"已暂停", style="red")
        if project.project_status == 0
        else Text(f"进行中", style="green")
    )

    grid = Table.grid(padding=1)
    grid.add_row(
        Text(
            f"{project.project_name}: {rest}/{project.fatigue_total}:",
            style="bold blue",
        ),
        status,
        bar,
        Text(f"{rest/project.fatigue_total:.1%}", style="magenta"),
    )
    console.print(grid)


def list_all():
    results = get_today_accumulated_fatigue()
    console = Console()
    if not results:
        typer.secho("没有记录，请确认", fg=typer.colors.RED, err=True)
    grid = Table.grid(padding=1)
    grid.add_column(justify="right")
    grid.add_column(justify="left")
    grid.add_column(justify="right")
    for r in results:
        rest = int(r["total"] - r["used"])
        if rest < 0:
            rest = 0

        bar = ProgressBar(
            total=r["total"],
            completed=rest,
            width=40,
            complete_style="yellow",
            finished_style="green",
        )
        status = (
            Text(f"已暂停", style="red")
            if r["status"] == 0
            else Text(f"进行中", style="green")
        )
        # 2. 把你要展示的东西，作为“一行”加进去
        grid.add_row(
            Text(
                f"{r['name']}: {rest}/{r['total']}:",
                style="bold blue",
            ),
            status,
            bar,
            Text(f"{rest/r['total']:.1%}", style="magenta"),
        )
    console.print(grid)


# def show(project_name):
#     console = Console()
#     project = get_project_name(project_name=project_name)

#     if not project:
#         print("找不到此项目")
#         return

#     record = get_recent_pending_record_by_project(
#         Record(id=None, project_id=project.id)
#     )
#     achive_duration_mins = int(get_today_accumulated_fatigue() / 60)
#     print("已经消耗的分钟数", achive_duration_mins)
#     costed_rate_fatigue = int(achive_duration_mins * project.fatigue_rate)
#     total = project.fatigue_total
#     if not record:
#         cost = costed_rate_fatigue
#     else:
#         if record.stop != -1:
#             stop_time = record.stop
#         else:
#             stop_time = int(datetime.now().timestamp())
#         start_time = record.start
#         duration_time = stop_time - start_time
#         duration_mins = int(duration_time / 60)
#         cost = int(duration_mins * project.fatigue_rate) + costed_rate_fatigue
#         if cost > total:
#             cost = total

#     completed = total - cost

#     bar = ProgressBar(
#         total=total,
#         completed=completed,
#         width=40,
#         complete_style="yellow",
#         finished_style="green",
#     )

#     grid = Table.grid(padding=1)

#     # 2. 把你要展示的东西，作为“一行”加进去
#     grid.add_row(
#         Text(f"{project.project_name}: {completed}/{total}", style="bold blue"),
#         bar,
#         Text(f"{completed/total:.1%}", style="magenta"),
#     )
#     console.print(grid)



import typer
from package.service.record.start import start
from package.service.record.stop import stop
from package.service.record.show import show, list_all
from package.service.record.complete import complete_project_name
from package.service.project.add import add_project

from typing_extensions import Annotated

app = typer.Typer(help="mana")


@app.callback(invoke_without_command=True)
def main(ctx: typer.Context):
    if ctx.invoked_subcommand is None:
        list_all()


# def start(task_desc, project_name, extra_id):
@app.command()
def open(
    project: Annotated[
        str, typer.Argument(help="project", shell_complete=complete_project_name)
    ],
    task_desc: Annotated[str, typer.Option("--task", "-t", help="task")] = "",
    extra_id: Annotated[str, typer.Option("--extra", "-e", help="extra_id")] = "",
):
    start(project_name=project, task_desc=task_desc, extra_id=extra_id)


@app.command()
def close(
    project: Annotated[
        str, typer.Argument(help="project", shell_complete=complete_project_name)
    ],
):
    stop(project_name=project)


@app.command()
def check(
    project: Annotated[
        str, typer.Argument(help="project", shell_complete=complete_project_name)
    ],
):
    show(project)


@app.command()
def checkall():
    list_all()


@app.command()
def add(
    project: Annotated[str, typer.Argument(help="project")],
    total: Annotated[int, typer.Option("--total", "-t", help="task")] = 156,
    rate: Annotated[float, typer.Option("--extra", "-r", help="extra_id")] = 1.0,
):
    add_project(project_name=project, total=total, rate=rate)
